@@ -90,103 +90,13 @@ single signature, regardeless of its size.
 
 =cut
 
-sub signature {
-    my $self = shift;
-    if (@_ == 1) {
-        $self->{signature} = shift;
-    }
-    my $sig = $self->{signature} || 0;
-    return $self->_optimize_signature($sig) + 0; # force the scalar context
-}
+sub pages_per_sheet { 4 };
+
+=head2 INTERNALS
 
 =head3 pages_per_sheet
 
-The number of logical pages which fit on a sheet, recto-verso. For
-this class, it will always return 4. Subclasses are allowed to change
-this.
-
-=cut
-
-sub pages_per_sheet {
-    return shift->{page_per_sheet} || 4;
-}
-
-sub _set_pages_per_sheet {
-    # private
-    my ($self, $num) = @_;
-    die "bad usage" unless $num;
-    if ($num == 2 or $num == 4 or $num == 8 or $num == 16 or $num == 32) {
-        $self->{page_per_sheet} = $num;
-    }
-    else {
-        die "bad number";
-    }
-    return $self->pages_per_sheet;
-}
-
-sub _optimize_signature {
-    my ($self, $sig, $total_pages) = @_;
-    unless ($total_pages) {
-        $total_pages = $self->total_pages;
-    }
-    return 0 unless $sig;
-    my $ppsheet = $self->pages_per_sheet or die;
-    if ($sig =~ m/^[0-9]+$/s) {
-        die "Signature must be a multiple of $ppsheet" if $sig % $ppsheet;
-        return $sig;
-    }
-    my ($min, $max);
-    if ($sig =~ m/^([0-9]+)?-([0-9]+)?$/s) {
-        $min = $1 || $ppsheet;
-        $max = $2 || $total_pages;
-        $min = $min + (($ppsheet - ($min % $ppsheet)) % $ppsheet);
-        $max = $max + (($ppsheet - ($max % $ppsheet)) % $ppsheet);
-        die "Bad range $max - $min" unless $max > $min;
-        die "bad min $min" if $min % $ppsheet;
-        die "bad max $max" if $max % $ppsheet;
-    }
-    else {
-        die "Unrecognized range $sig";
-    }
-    my $signature = 0;
-    my $roundedpages = $total_pages + (($ppsheet - ($total_pages % $ppsheet)) % $ppsheet);
-    my $needed = $roundedpages - $total_pages;
-    die "Something is wrong" if $roundedpages % $ppsheet;
-    if ($roundedpages <= $min) {
-        wantarray ? return ($roundedpages, $needed) : return $roundedpages;
-    }
-    $signature = $self->_find_signature($roundedpages, $max);
-    if ($roundedpages > $max) {
-        while ($signature < $min) {
-            $roundedpages += $ppsheet;
-            $needed += $ppsheet;
-            $signature = $self->_find_signature($roundedpages, $max)
-        }
-    }
-    # warn "Needed $needed blank pages";
-    wantarray ? return ($signature, $needed) : return $signature;
-}
-
-sub _find_signature {
-    my ($self, $num, $max) = @_;
-    my $ppsheet = $self->pages_per_sheet or die;
-    die "not a multiple of $ppsheet" if $num % $ppsheet;
-    die "uh?" unless $num;
-    my $i = $max;
-    while ($i > 0) {
-        # check if the the pagenumber is divisible by the signature
-        # with modulo 0
-        # warn "trying $i for $num / max $max\n";
-        if (($num % $i) == 0) {
-            return $i;
-        }
-        $i -= $ppsheet;
-    }
-    warn "Looped ended with no result\n";
-}
-
-
-=head2 Internal (but documented) methods
+Always return 4.
 
 =head3 page_sequence_for_booklet($pages, $signature)
 
@@ -198,23 +108,13 @@ PDF->PS->PDF route.
 
 sub page_sequence_for_booklet {
     my ($self, $pages, $signature) = @_;
-    unless (defined $pages) {
-        $pages = $self->total_pages;
-    }
-    unless (defined $signature) {
-        $signature = $self->signature;
-    }
-    my (@pgs, $maxpage);
-    use integer;
-    if (!$signature) {
-        # rounding
-        my $ppsheet = $self->pages_per_sheet;
-        $signature = $maxpage =
-          $pages + (($ppsheet - ($pages % $ppsheet)) % $ppsheet);
-    }
-    else {
-        $maxpage = $pages + (($signature - ($pages % $signature)) % $signature)
-    }
+    # if set to 0, get the actual number
+    $signature ||= $self->computed_signature;
+    $pages ||= $self->total_pages;
+    my $maxpage = $self->total_output_pages;
+    my @pgs;
+    {
+        use integer;
     for (my $currentpg = 0; $currentpg < $maxpage; $currentpg++) {
         my $actualpg = $currentpg - ($currentpg % $signature);
         my $modulo = $currentpg % 4;
@@ -231,6 +131,7 @@ sub page_sequence_for_booklet {
         }
         push @pgs, $actualpg;
     }
+    }
     my @out;
     # if we want a cover, we need to find the index of the last page,
     # and the first undef page, which could be at the beginning of the
